{-# LANGUAGE OverloadedStrings #-}
module Main where

import FORP
import Formula 

import Control.Monad (liftM)
import System.Directory (getDirectoryContents)
import System.Environment(getArgs)
import System.TimeIt(timeIt)


main :: IO ()
main = do
  as <- getArgs
  case as of
    [] -> examples >>= testExamples
    (cmd:xs) 
      | isCmd cmd -> proveProblems cmd xs
      | otherwise -> testExamples (cmd:xs)

proveProblems :: String -> [FilePath] -> IO ()
proveProblems cmd = mapM_ (proveProblem cmd)

proveProblem :: String -> FilePath -> IO ()
proveProblem cmd p = do
  cls <- parse p
  let 
    (pp,mkSty) = interpret cmd
    sty        = mkSty cls
  print $ "problem: " ++ p
  print cls
  putStrLn ">>>"
  timeIt (print $! proveWith2 pp sty cls)
  putStrLn "<<<"

examples :: IO [FilePath]
examples = do
  es <- getDirectoryContents "./examples1/"
  return $ map ("./examples1/"++) $ filter (`notElem` [".",".."]) es

parse :: String -> IO Clauses
parse s = parseProblem `liftM` readFile s

testExamples :: [FilePath] -> IO()
testExamples = mapM_ testExample

testExample :: FilePath -> IO()
testExample ex = do
  cls <- parse ex
  print $ "problem: " ++ ex
  print cls
  putStrLn "start"
  print $ "problem: " ++ ex
  print cls
  if isParaProblem cls then paranormal cls else normal cls
  putStrLn "exit"
  where
    paranormal cls = do
      putStrLn "---"
      timeIt (print2 "ord"     $! proveWith ((snd $ interpret "none**presolutionSty") cls) cls)
      putStrLn "---"
      timeIt (print2 "ord+std" $! proveWith ((snd $ interpret "some**presolutionSty") cls) cls)
      putStrLn "---"
      timeIt (print2 "ord"     $! proveWith ((snd $ interpret "none**pfwdSubsumptionSty") cls) cls)
      putStrLn "---"
      timeIt (print2 "ord+std" $! proveWith ((snd $ interpret "some**pfwdSubsumptionSty") cls) cls)
      putStrLn "---"
    normal cls = do
      putStrLn "---"
      timeIt (print2 "ord"     $! proveWith ((snd $ interpret "none**resolutionSty") cls) cls)
      putStrLn "---"
      timeIt (print2 "ord+std" $! proveWith ((snd $ interpret "some**resolutionSty") cls) cls)
      putStrLn "---"
      timeIt (print2 "fwd"     $! proveWith ((snd $ interpret "none**fwdSubsumptionSty") cls) cls)
      putStrLn "---"
      timeIt (print2 "ord+fwd" $! proveWith ((snd $ interpret "some**fwdSubsumptionSty") cls) cls)
      putStrLn "---"
      timeIt (print2 "bwd"     $! proveWith ((snd $ interpret "none**bwdSubsumptionSty") cls) cls)
      putStrLn "---"
      timeIt (print2 "ord+bwd" $! proveWith ((snd $ interpret "some**bwdSubsumptionSty") cls) cls)

      putStrLn "---"
      timeIt (print2 "rpl"     $! proveWith ((snd $ interpret "none**replacementSty") cls) cls)
      putStrLn "---"
      timeIt (print2 "ord+rpl" $! proveWith ((snd $ interpret "some**replacementSty") cls) cls)
      putStrLn "---"
    print2 a b = putStrLn a >> print b


